/*	vmusave.cpp	-- ljsdcdev
	Dreamcast specific saving routines
	ljsdcdev
	(c) 2003 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* This file handles saving on Dreamcast ONLY */
#ifndef _arch_dreamcast
#error This file is for Dreamcast Only
#endif

#include "../../ljsdcdev.h"

void save_data(save_data_t * save, SDL_Surface * screen)	{
	uint8 * data;
	file_t fd;
	vmu_pkg_t pkg;
	uint8 * pkg_out;
	int pkg_size;
	uint8 * comp;
	uint32 len;

	/* Get the data into an array of uint8's */
	data = (uint8 *) malloc(sizeof(save_data_t));
	memcpy(data, save, sizeof(save_data_t));

	/* Make a buffer */
	len = sizeof(save_data_t)+660;
	comp = (uint8 *) malloc(len);

	/* Compress the data */
	int err = compress2(comp, &len, data, sizeof(save_data_t), 9);

	if(err == Z_MEM_ERROR)	{
		SDL_Rect rect;
		rect = rect_set(640, 480, 0, 0);
		SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format, options.mc.r, options.mc.g, options.mc.b));
		sge_tt_textout(screen, font, "Out of Memory", 32, 32, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0xFF, SDL_ALPHA_OPAQUE);
		SDL_Flip(screen);
		SDL_Delay(5000);
		return;
	}
	else if(err == Z_BUF_ERROR)	{
		SDL_Rect rect;
		rect = rect_set(640, 480, 0, 0);
		SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format, options.mc.r, options.mc.g, options.mc.b));
		sge_tt_textout(screen, font, "Out of Buffer", 32, 32, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0xFF, SDL_ALPHA_OPAQUE);
		SDL_Flip(screen);
		SDL_Delay(5000);
		return;
	}
	else if(err == Z_STREAM_ERROR)	{
		SDL_Rect rect;
		rect = rect_set(640, 480, 0, 0);
		SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format, options.mc.r, options.mc.g, options.mc.b));
		sge_tt_textout(screen, font, "Stream Error", 32, 32, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0xFF, SDL_ALPHA_OPAQUE);
		SDL_Flip(screen);
		SDL_Delay(5000);
		return;
	}

	/* Set up the vars in the header */
	strcpy(pkg.desc_short, "ljsdcdev");
	strcpy(pkg.desc_long, "ljsdcdev S1");
	strcpy(pkg.app_id, "ljsdcdev");
	pkg.icon_cnt = 0;
	pkg.icon_anim_speed = 0;
	pkg.eyecatch_type = VMUPKG_EC_NONE;
	pkg.data_len = len;
	pkg.data = comp;

	/* Build the package */
	vmu_pkg_build(&pkg, &pkg_out, &pkg_size);

	/* Just for safety */
	fs_unlink("/vmu/a1/ljsdcdev");

	/* Now write it */
	fd = fs_open("/vmu/a1/ljsdcdev", O_WRONLY);
	fs_write(fd, pkg_out, pkg_size);

	/* Clean up */
	fs_close(fd);
	free(pkg_out);
	free(data);
	free(comp);
}

void read_data(save_data_t * save, SDL_Surface * screen)	{
	uint8 * raw_data;
	uint32 raw_size;
	uint8 * comp_data;
	uint32 comp_size;
	uint8 * real_data;
	uint32 real_size;
	vmu_pkg_t pkg;
	file_t fd;

	/* Open it */
	fd = fs_open("/vmu/a1/ljsdcdev", O_RDONLY);
	if(!fd)	{
		SDL_Rect rect;
		rect = rect_set(640, 480, 0, 0);
		SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format, options.mc.r, options.mc.g, options.mc.b));
		sge_tt_textout(screen, font, "Error Opening VMU File", 32, 32, options.tc.r, options.tc.g, options.tc.b, options.mc.r, options.mc.g, options.mc.b, SDL_ALPHA_OPAQUE);
		SDL_Flip(screen);
		SDL_Delay(5000);
		return;
	}

	/* Get the size */
	raw_size = fs_total(fd);
	if(raw_size == 0)	{
		SDL_Rect rect;
		rect = rect_set(640, 480, 0, 0);
		SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format, options.mc.r, options.mc.g, options.mc.b));
		sge_tt_textout(screen, font, "Zero Length VMU File", 32, 32, options.tc.r, options.tc.g, options.tc.b, options.mc.r, options.mc.g, options.mc.b, SDL_ALPHA_OPAQUE);
		SDL_Flip(screen);
		SDL_Delay(5000);
		return;
	}

	/* Read the data into a buffer */
	raw_data = (uint8 *) malloc(raw_size);
	fs_read(fd, raw_data, raw_size);

	/* Parse the package, move the data elsewhere */
	vmu_pkg_parse(raw_data, &pkg);
	comp_size = pkg.data_len;
	comp_data = (uint8 *) malloc(comp_size);
	memcpy(comp_data, pkg.data, comp_size);
	real_size = sizeof(save_data_t);
	real_data = (uint8 *) malloc(real_size);

	/* Uncompress the data */
	int err = uncompress(real_data, &real_size, comp_data, comp_size);
	/* Here's all the terrible stuff that can go wrong */
	if(err == Z_MEM_ERROR)	{
		SDL_Rect rect;
		rect = rect_set(640, 480, 0, 0);
		SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format, options.mc.r, options.mc.g, options.mc.b));
		sge_tt_textout(screen, font, "Out of Memory", 32, 32, options.tc.r, options.tc.g, options.tc.b, options.mc.r, options.mc.g, options.mc.b, SDL_ALPHA_OPAQUE);
		SDL_Flip(screen);
		SDL_Delay(5000);
		return;
	}
	else if(err == Z_BUF_ERROR)	{
		SDL_Rect rect;
		rect = rect_set(640, 480, 0, 0);
		SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format, options.mc.r, options.mc.g, options.mc.b));
		sge_tt_textout(screen, font, "Out of Buffer", 32, 32, options.tc.r, options.tc.g, options.tc.b, options.mc.r, options.mc.g, options.mc.b, SDL_ALPHA_OPAQUE);
		SDL_Flip(screen);
		SDL_Delay(5000);
		return;
	}
	else if(err == Z_DATA_ERROR)	{
		SDL_Rect rect;
		rect = rect_set(640, 480, 0, 0);
		SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format, options.mc.r, options.mc.g, options.mc.b));
		sge_tt_textout(screen, font, "Data Corrupted", 32, 32, options.tc.r, options.tc.g, options.tc.b, options.mc.r, options.mc.g, options.mc.b, SDL_ALPHA_OPAQUE);
		SDL_Flip(screen);
		SDL_Delay(5000);
		return;
	}

	/* If we get here, all is safe, copy the data */
	memcpy(save, real_data, real_size);

	/* Clean Up */
	fs_close(fd);
	free(raw_data);
	free(comp_data);
	free(real_data);
}
