/* 	save.cpp -- ljsdcdev
	Saving/loading utility function
	ljsdcdev
	(c) 2003 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ljsdcdev.h"

/* Convert from stat_data_t to stats_data_t */
stats_data_t stat_to_stats(stat_data_t src)	{
	stats_data_t dst;

	/* name */
	strcpy(dst.name, src.name);

	/* stats */
	dst.stats.attack = src.stats.attack;
	dst.stats.defense = src.stats.defense;
	dst.stats.speed = src.stats.speed;
	dst.stats.strength = src.stats.strength;
	dst.stats.mdef = src.stats.mdef;
	dst.stats.matt = src.stats.matt;
	dst.stats.intel = src.stats.intel;
	dst.stats.accuracy = src.stats.accuracy;
	dst.stats.hp = src.stats.hp;
	dst.stats.hp_rem = src.stats.hp_rem;
	dst.stats.mp = src.stats.mp;
	dst.stats.mp_rem = src.stats.mp_rem;
	dst.stats.level = src.stats.level;
	dst.stats.exp = src.stats.exp;
	dst.stats.exprem = src.stats.exprem;
	dst.stats.rank = src.stats.rank;

	/* magic */
	dst.magic.m11 = src.magic.m11;
	dst.magic.m12 = src.magic.m12;
	dst.magic.m21 = src.magic.m21;
	dst.magic.m22 = src.magic.m22;
	dst.magic.m31 = src.magic.m31;
	dst.magic.m32 = src.magic.m32;
	dst.magic.m41 = src.magic.m41;
	dst.magic.m42 = src.magic.m42;

	/* skill */
	dst.skills.skill01 = src.skills.skill01;
	dst.skills.skill02 = src.skills.skill02;
	dst.skills.skill03 = src.skills.skill03;
	dst.skills.skill04 = src.skills.skill04;
	dst.skills.skill05 = src.skills.skill05;

	/* equipment */
	dst.equip.weap = src.equip.weap;
	dst.equip.armor = src.equip.armor;
	dst.equip.head = src.equip.head;
	dst.equip.foot = src.equip.foot;
	dst.equip.acc = src.equip.acc;

	/* Battle statistics */
	dst.bat.kills = src.bat.kills;
	dst.bat.defeats = src.bat.defeats;

	return dst;
}

/* Convert from stats_data_t to stat_data_t */
stat_data_t stats_to_stat(stats_data_t src)	{
	stat_data_t dst;

	/* name */
	strcpy(dst.name, src.name);

	/* stats */
	dst.stats.attack = src.stats.attack;
	dst.stats.defense = src.stats.defense;
	dst.stats.speed = src.stats.speed;
	dst.stats.strength = src.stats.strength;
	dst.stats.mdef = src.stats.mdef;
	dst.stats.matt = src.stats.matt;
	dst.stats.intel = src.stats.intel;
	dst.stats.accuracy = src.stats.accuracy;
	dst.stats.hp = src.stats.hp;
	dst.stats.hp_rem = src.stats.hp_rem;
	dst.stats.mp = src.stats.mp;
	dst.stats.mp_rem = src.stats.mp_rem;
	dst.stats.level = src.stats.level;
	dst.stats.exp = src.stats.exp;
	dst.stats.exprem = src.stats.exprem;
	dst.stats.rank = src.stats.rank;

	/* magic */
	dst.magic.m11 = src.magic.m11;
	dst.magic.m12 = src.magic.m12;
	dst.magic.m21 = src.magic.m21;
	dst.magic.m22 = src.magic.m22;
	dst.magic.m31 = src.magic.m31;
	dst.magic.m32 = src.magic.m32;
	dst.magic.m41 = src.magic.m41;
	dst.magic.m42 = src.magic.m42;

	/* skill */
	dst.skills.skill01 = src.skills.skill01;
	dst.skills.skill02 = src.skills.skill02;
	dst.skills.skill03 = src.skills.skill03;
	dst.skills.skill04 = src.skills.skill04;
	dst.skills.skill05 = src.skills.skill05;

	/* equipment */
	dst.equip.weap = src.equip.weap;
	dst.equip.armor = src.equip.armor;
	dst.equip.head = src.equip.head;
	dst.equip.foot = src.equip.foot;
	dst.equip.acc = src.equip.acc;

	/* other stuff */
	dst.ally = 1;

	/* Image Loading */
	if(dst.stats.rank == 0)	{
		dst.images.splayer = IMG_Load(P1);
	}
	else if(dst.stats.rank == 1)	{
		dst.images.splayer = IMG_Load(P2);
	}
	else if(dst.stats.rank == 2)	{
		dst.images.splayer = IMG_Load(P3);
	}
	else if(dst.stats.rank == 3)	{
		dst.images.splayer = IMG_Load(P4);
	}

	/* Battle statistics */
	dst.bat.kills = src.bat.kills;
	dst.bat.defeats = src.bat.defeats;

	return dst;
}

void save_game(SDL_Surface * screen)	{
	save_data_t save;

	/* put our stats in */
	save.stats.p1 = stat_to_stats(stats[0]);
	save.stats.p2 = stat_to_stats(stats[1]);
	save.stats.p3 = stat_to_stats(stats[2]);
	save.stats.p4 = stat_to_stats(stats[3]);

	/* map stuffs */
	save.map.x = posx;
	save.map.y = posy;
	save.map.num = areanum;

	/* misc. */
	int i;
	for(i = 0; i < 50; i++)
		save.items[i] = items[i];
	save.time = 0;
	save.gold = 0;
	save.opts = options;

	/* save it */
	save_data(&save, screen);
}

void load_game(SDL_Surface * screen)	{
	save_data_t save;

	/* read it */
	read_data(&save, screen);

	/* put our stats in */
	stats[0] = stats_to_stat(save.stats.p1);
	stats[1] = stats_to_stat(save.stats.p2);
	stats[2] = stats_to_stat(save.stats.p3);
	stats[3] = stats_to_stat(save.stats.p4);

	/* map stuffs */
	posx = save.map.x;
	posy = save.map.y;
	areanum = save.map.num;

	/* misc. */
	int i;
	for(i = 0; i < 50; i++)
		items[i] = save.items[i];
	options = save.opts;
}
