/*	tilemap.cpp -- ljsdcdev
	SDLMappy Interface
	ljsdcdev
	(c) 2003 Lawrence Sebald

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ljsdcdev.h"
#include "SDLMappy.h"
#include "areas.h"

SDLMappy * tilemap;
static SDL_Rect prect;
int posx = 0, posy = 0;
static int sposx = 0, sposy = 0;
static int tm_done = 0;
int areanum;
static int num_en;
static int en_num;
static stat_data_t blank;
Mix_Music * tmmusic;
/* For the new random battle determiner */
static int area_freq, area_min, steps;
static int area_menem;

/* Random battle determiner, as recomended by MrSiggler */
static inline void tilemap_battle(SDL_Surface * screen)	{
	int i;
	steps--;
	if(steps == 0)	{
		Mix_HaltMusic();
		clear_screen(screen);
		do_battle(screen);
		sge_Randomize();
		num_en = sge_Random(0, area_menem);
		for(i = 0; i <= 8; i++)	{
			enemies[i] = blank;
		}
		for(i = 0; i <= num_en; i++)	{
			en_num = sge_Random(0, 9);
			enemies[i] = *area.enemy[en_num];
		}
		steps = sge_Random(0, area_freq) + area_min;
		direct = NONE;
		Mix_PlayMusic(tmmusic, -1);
	}
}

inline static void tilemap_scroll()	{
	int bw = tilemap->GetMapBlockWidth();
	int bh = tilemap->GetMapBlockHeight();

	if(sposx > 480 && tilemap->MapGetXPosition() != tilemap->GetMapWidth())	{
		sposx -= 32;
		tilemap->MapMoveTo(tilemap->MapGetXPosition() + bw, tilemap->MapGetYPosition());
	}
	if(sposy > 320 && tilemap->MapGetYPosition() != tilemap->GetMapHeight())	{
		sposy -= 32;
		tilemap->MapMoveTo(tilemap->MapGetXPosition(), tilemap->MapGetYPosition() + bh);
	}
	if(sposx < 160 && tilemap->MapGetXPosition() != 0)	{
		sposx += 32;
		tilemap->MapMoveTo(tilemap->MapGetXPosition() - bw, tilemap->MapGetYPosition());
	}
	if(sposy < 160 && tilemap->MapGetYPosition() != 0)	{
		sposy += 32;
		tilemap->MapMoveTo(tilemap->MapGetXPosition(), tilemap->MapGetYPosition() - bh);
	}
}

static void tilemap_draw_player(SDL_Surface * screen)	{
	BLKSTR * block;
	if(sposx >= 0 && sposy >= 0)	{
		/* Assume that all anims are not passable by foot
		   since the thing crashes if we do a MapGetBlockInPixels on an anim! */
		if(tilemap->MapGetTileInPixels(posx, posy) > 0)	{
			block = tilemap->MapGetBlockInPixels(posx, posy);
			if(!block->tl)	{
				tilemap_scroll();
				prect = rect_set(32, 32, sposx, sposy);
			}
		}
	}
	sposx = prect.x;
	sposy = prect.y;
	posx = tilemap->MapGetXPosition() + sposx;
	posy = tilemap->MapGetYPosition() + sposy;
	SDL_BlitSurface(stats[0].images.splayer, NULL, screen, &prect);
}

static inline void tilemap_init()	{
	tilemap = new SDLMappy();
	tilemap->LoadMap(TESTMAP, 0, 0, 640, 480);
	tilemap->MapInitAnims();
}

static void tilemap_mainloop(SDL_Surface * screen, SDL_Surface * subscreen)	{
	int counter = 0, rand_needed = 0;
	SDL_Event event;
	while(tm_done != 1)	{
		rand_needed = 0;
		tilemap->MapUpdateAnims();
		tilemap->MapChangeLayer(0);
		tilemap->MapDrawBGT(subscreen);
		tilemap_draw_player(subscreen);
		SDL_BlitSurface(subscreen, NULL, screen, NULL);
		SDL_Flip(screen);
		SDL_PollEvent(&event);
		switch(event.type)	{
			case SDL_KEYUP:
				switch(event.key.keysym.sym)	{
					case SDLK_DOWN:
					case SDLK_UP:
					case SDLK_RIGHT:
					case SDLK_LEFT:
						direct = NONE;
						break;
				}
				break;
			case SDL_KEYDOWN:
				switch(event.key.keysym.sym)	{
					case SDLK_DOWN:
						posy += 32;
						sposy += 32;
						rand_needed = 1;
						direct = DOWN;
						break;
					case SDLK_UP:
						posy -= 32;
						sposy -= 32;
						rand_needed = 1;
						direct = UP;
						break;
					case SDLK_RIGHT:
						posx += 32;
						sposx += 32;
						rand_needed = 1;
						direct = RIGHT;
						break;
					case SDLK_LEFT:
						posx -= 32;
						sposx -= 32;
						rand_needed = 1;
						direct = LEFT;
						break;
					case SDLK_ESCAPE:
						Mix_HaltMusic();
						menu_do_menu(screen);
						SDL_Delay(100);
						Mix_PlayMusic(tmmusic, -1);
						break;
				}
				break;

			case SDL_JOYHATMOTION:
				if(event.jhat.value == SDL_HAT_UP)	{
					posy -= 32;
					sposy -= 32;
					rand_needed = 1;
					direct = UP;
				}
				if(event.jhat.value == SDL_HAT_DOWN)	{
					posy += 32;
					sposy += 32;
					rand_needed = 1;
					direct = DOWN;
				}
				if(event.jhat.value == SDL_HAT_RIGHT)	{
					posx += 32;
					sposx += 32;
					rand_needed = 1;
					direct = RIGHT;
				}
				if(event.jhat.value == SDL_HAT_LEFT)	{
					posx -= 32;
					sposx -= 32;
					rand_needed = 1;
					direct = LEFT;
				}
				break;

			case SDL_JOYAXISMOTION:
				if(event.jaxis.axis == 0)	{
					if(event.jaxis.value > joyval)	{
						posx += 32;
						sposx += 32;
						rand_needed = 1;
						direct = RIGHT;
					}
					if(event.jaxis.value < -joyval)	{
						posx -= 32;
						sposx -= 32;
						rand_needed = 1;
						direct = LEFT;
					}
				}
				if(event.jaxis.axis == 1)	{
					if(event.jaxis.value > joyval)	{
						posy += 32;
						sposy += 32;
						rand_needed = 1;
						direct = DOWN;
					}
					if(event.jaxis.value < -joyval)	{
						posy -= 32;
						sposy -= 32;
						rand_needed = 1;
						direct = UP;
					}
				}
				break;
			case SDL_JOYBUTTONDOWN:
				if(event.jbutton.button == 4)	{
					Mix_HaltMusic();
					menu_do_menu(screen);
					SDL_Delay(100);
					Mix_PlayMusic(tmmusic, -1);
				}
				break;
		}
		if(counter == 60)	{
			if(direct == DOWN)	{
				sposy++;
				posy++;
				rand_needed = 1;
			}
			if(direct == UP)	{
				sposy--;
				posy--;
				rand_needed = 1;
			}
			if(direct == LEFT)	{
				sposx--;
				posx--;
				rand_needed = 1;
			}
			if(direct == RIGHT)	{
				sposx++;
				posx++;
				rand_needed = 1;
			}
			counter = 0;
		}

		if(rand_needed == 1)	{
			tilemap_battle(screen);
		}
		SDL_Delay(30);
		counter++;
	}
}

void do_tilemap(SDL_Surface * screen)	{
	tmmusic = Mix_LoadMUS(TILMUS);
	Mix_PlayMusic(tmmusic, -1);
	SDL_Surface * subscreen;
	subscreen = SDL_AllocSurface(SDL_SWSURFACE, 640, 480, 16, 0, 0, 0, 0);
	clear_screen(subscreen);
	clear_screen(screen);
	tilemap_init();
	BLKSTR * block1 = tilemap->MapGetBlock(0, 0);
	areanum = (int) block1->user5;
	area_freq = (int) block1->user1;
	area_min = (int) block1->user2;
	area_menem = (int) block1->user3;
	sge_Randomize();
	steps = sge_Random(0, area_freq) + area_min;
	area_load_enem(areanum);
	sge_Randomize();
	num_en = sge_Random(0, area_menem);
	int i;
	for(i = 0; i <= num_en; i++)	{
		en_num = sge_Random(0, 9);
		enemies[i] = *area.enemy[en_num];
	}
	tilemap_mainloop(screen, subscreen);
}
